@@ -9,13 +9,14 @@ A lightweight terminal spinner.
 """
 
 import contextlib
-import datetime
 import functools
 import itertools
 import signal
 import sys
 import threading
 import time
+import warnings
+from datetime import timedelta
 from typing import List, Set, Union
 
 from termcolor import colored
@@ -75,6 +76,7 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
         self._last_frame = None
         self._stdout_lock = threading.Lock()
         self._hidden_level = 0
+        self._cur_line_len = 0
 
         # Signals
 
@@ -92,7 +94,7 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
     # Dunders
     #
     def __repr__(self):
-        return "<Yaspin frames={0!s}>".format(self._frames)
+        return f"<Yaspin frames={self._frames!s}>"
 
     def __enter__(self):
         self.start()
@@ -134,9 +136,7 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
         # Common error for unsupported attributes
         else:
             raise AttributeError(
-                "'{0}' object has no attribute: '{1}'".format(
-                    self.__class__.__name__, name
-                )
+                f"'{self.__class__.__name__}' object has no attribute: '{name}'"
             )
         return self
 
@@ -225,9 +225,7 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
         if self._sigmap:
             self._register_signal_handlers()
 
-        if sys.stdout.isatty():
-            self._hide_cursor()
-
+        self._hide_cursor()
         self._start_time = time.time()
         self._stop_time = None  # Reset value to properly calculate subsequent spinner starts (if any)  # pylint: disable=line-too-long
         self._stop_spin = threading.Event()
@@ -251,11 +249,8 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
             self._stop_spin.set()
             self._spin_thread.join()
 
-        sys.stdout.write("\r")
         self._clear_line()
-
-        if sys.stdout.isatty():
-            self._show_cursor()
+        self._show_cursor()
 
     def hide(self):
         """Hide the spinner to allow for custom writing to the terminal."""
@@ -265,9 +260,6 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
             with self._stdout_lock:
                 # set the hidden spinner flag
                 self._hide_spin.set()
-
-                # clear the current line
-                sys.stdout.write("\r")
                 self._clear_line()
 
                 # flush the stdout buffer so the current line
@@ -298,7 +290,6 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
                 self._hide_spin.clear()
 
                 # clear the current line so the spinner is not appended to it
-                sys.stdout.write("\r")
                 self._clear_line()
 
     def write(self, text):
@@ -306,7 +297,6 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
         # similar to tqdm.write()
         # https://pypi.python.org/pypi/tqdm#writing-messages
         with self._stdout_lock:
-            sys.stdout.write("\r")
             self._clear_line()
 
             if isinstance(text, (str, bytes)):
@@ -317,7 +307,8 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
             # Ensure output is Unicode
             assert isinstance(_text, str)
 
-            sys.stdout.write("{0}\n".format(_text))
+            sys.stdout.write(f"{_text}\n")
+            self._cur_line_len = 0
 
     def ok(self, text="OK"):
         """Set Ok (success) finalizer to a spinner."""
@@ -332,6 +323,13 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
     #
     # Protected
     #
+    @staticmethod
+    def _warn_color_disabled():
+        warnings.warn(
+            "color, on_color and attrs are not supported when running in jupyter",
+            stacklevel=3,
+        )
+
     def _freeze(self, final_text):
         """Stop spinner, compose last frame and 'freeze' it."""
         text = to_unicode(final_text)
@@ -342,6 +340,7 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
         self.stop()
         with self._stdout_lock:
             sys.stdout.write(self._last_frame)
+            self._cur_line_len = 0
 
     def _spin(self):
         while not self._stop_spin.is_set():
@@ -357,14 +356,19 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
 
             # Write
             with self._stdout_lock:
-                sys.stdout.write(out)
                 self._clear_line()
+                sys.stdout.write(out)
                 sys.stdout.flush()
+                self._cur_line_len = max(self._cur_line_len, len(out))
 
             # Wait
             self._stop_spin.wait(self._interval)
 
     def _compose_color_func(self):
+        if self.is_jupyter():
+            # ANSI Color Control Sequences are problematic in Jupyter
+            return None
+
         return functools.partial(
             colored,
             color=self._color,
@@ -389,13 +393,15 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
 
         if self._timer:
             sec, fsec = divmod(round(100 * self.elapsed_time), 100)
-            text += " ({}.{:02.0f})".format(datetime.timedelta(seconds=sec), fsec)
+            text += " ({}.{:02.0f})".format(  # pylint: disable=consider-using-f-string
+                timedelta(seconds=sec), fsec
+            )
 
         # Mode
         if not mode:
-            out = "\r{0} {1}".format(frame, text)
+            out = f"\r{frame} {text}"
         else:
-            out = "{0} {1}\n".format(frame, text)
+            out = f"{frame} {text}\n"
 
         # Ensure output is Unicode
         assert isinstance(out, str)
@@ -406,7 +412,7 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
         # SIGKILL cannot be caught or ignored, and the receiving
         # process cannot perform any clean-up upon receiving this
         # signal.
-        if signal.SIGKILL in self._sigmap.keys():
+        if signal.SIGKILL in self._sigmap:
             raise ValueError(
                 "Trying to set handler for SIGKILL signal. "
                 "SIGKILL cannot be caught or ignored in POSIX systems."
@@ -438,12 +444,19 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
     #
     # Static
     #
+    @staticmethod
+    def is_jupyter() -> bool:
+        return not sys.stdout.isatty()
+
     @staticmethod
     def _set_color(value: str) -> str:
+        if Yaspin.is_jupyter():
+            Yaspin._warn_color_disabled()
+
         available_values = [k for k, v in COLOR_MAP.items() if v == "color"]
         if value not in available_values:
             raise ValueError(
-                "'{0}': unsupported color value. Use one of the: {1}".format(
+                "'{0}': unsupported color value. Use one of the: {1}".format(  # pylint: disable=consider-using-f-string
                     value, ", ".join(available_values)
                 )
             )
@@ -451,21 +464,27 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
 
     @staticmethod
     def _set_on_color(value: str) -> str:
+        if Yaspin.is_jupyter():
+            Yaspin._warn_color_disabled()
+
         available_values = [k for k, v in COLOR_MAP.items() if v == "on_color"]
         if value not in available_values:
             raise ValueError(
-                "'{0}': unsupported on_color value. "
+                "'{0}': unsupported on_color value. "  # pylint: disable=consider-using-f-string
                 "Use one of the: {1}".format(value, ", ".join(available_values))
             )
         return value
 
     @staticmethod
     def _set_attrs(attrs: List[str]) -> Set[str]:
+        if Yaspin.is_jupyter():
+            Yaspin._warn_color_disabled()
+
         available_values = [k for k, v in COLOR_MAP.items() if v == "attrs"]
         for attr in attrs:
             if attr not in available_values:
                 raise ValueError(
-                    "'{0}': unsupported attribute value. "
+                    "'{0}': unsupported attribute value. "  # pylint: disable=consider-using-f-string
                     "Use one of the: {1}".format(attr, ", ".join(available_values))
                 )
         return set(attrs)
@@ -486,7 +505,7 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
     def _set_side(side: str) -> str:
         if side not in ("left", "right"):
             raise ValueError(
-                "'{0}': unsupported side value. " "Use either 'left' or 'right'."
+                "'{0}': unsupported side value. Use either 'left' or 'right'."
             )
         return side
 
@@ -512,7 +531,7 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
             # Empty ``spinner.frames`` is handled by ``Yaspin._set_spinner``.
             # This code is very unlikely to be executed. However, it's still
             # here to be on a safe side.
-            raise ValueError("{0!r}: no frames found in spinner".format(spinner))
+            raise ValueError(f"{spinner!r}: no frames found in spinner")
 
         # Builtin ``reversed`` returns reverse iterator,
         # which adds unnecessary difficulty for returning
@@ -533,14 +552,22 @@ class Yaspin:  # pylint: disable=useless-object-inheritance,too-many-instance-at
 
     @staticmethod
     def _hide_cursor():
-        sys.stdout.write("\033[?25l")
-        sys.stdout.flush()
+        if sys.stdout.isatty():
+            # ANSI Control Sequence DECTCEM 1 does not work in Jupyter
+            sys.stdout.write("\033[?25l")
+            sys.stdout.flush()
 
     @staticmethod
     def _show_cursor():
-        sys.stdout.write("\033[?25h")
-        sys.stdout.flush()
+        if sys.stdout.isatty():
+            # ANSI Control Sequence DECTCEM 2 does not work in Jupyter
+            sys.stdout.write("\033[?25h")
+            sys.stdout.flush()
 
-    @staticmethod
-    def _clear_line():
-        sys.stdout.write("\033[K")
+    def _clear_line(self):
+        if sys.stdout.isatty():
+            # ANSI Control Sequence EL does not work in Jupyter
+            sys.stdout.write("\r\033[K")
+        else:
+            fill = " " * self._cur_line_len
+            sys.stdout.write(f"\r{fill}\r")
